// uxu_gg.h is a part of the VINCIA plug-in to the PYTHIA 8 generator.
// Copyright (C) 2017 P. Skands.
// VINCIA is licenced under the GNU GPL version 2.
// Please respect the MCnet Guidelines for Event Generator Authors and Users, 
//               http://www.montecarlonet.org/GUIDELINES

// This file was created on Tue 16 May 2017 14:36:53 AEST by ./convertMG5.sh
// VINCIA C++ wrapper example for user-defined uxu_gg.f

#ifndef VINCIA_uxu_gg_H
#define VINCIA_uxu_gg_H

#include "Vincia/MG5interface.h"

namespace Vincia {
using namespace Pythia8;

class MG5vincia31_uxu_gg : public MG5matrix {

 public:

  // Constructor
  MG5vincia31_uxu_gg() { initHelMap(); initColMap(); };

  // Destructor
  virtual ~MG5vincia31_uxu_gg() {};

  // Name
  virtual string name() {return "vincia31_uxu_gg";};

  // PDG ID label
  virtual vector< vector<int> > processList();

  // Number of legs
  virtual int nLegs() {return 4;}

  // Redefine methods to define/initialise helicity and colour maps
  virtual void initHelMap();
  virtual void initColMap();

  // Store momenta in MG5 format (from input in Pythia Vec4 format)
  // (Must be copied in derived classes so that pSav will be the
  // variable in the derived class, rather than the one in the base class)
  virtual void setMom(vector<Particle> pIn, bool permute=true) {
    for (int i=0; i<nLegs(); ++i) {
      if (permute) {
        pSav[i][0] = pIn[iVincia[i]].e();
        pSav[i][1] = pIn[iVincia[i]].px();
        pSav[i][2] = pIn[iVincia[i]].py();
        pSav[i][3] = pIn[iVincia[i]].pz();
      } else {
        pSav[i][0] = pIn[i].e();
        pSav[i][1] = pIn[i].px();
        pSav[i][2] = pIn[i].py();
        pSav[i][3] = pIn[i].pz();
      }
    }
  }

  // setMom with vector<Vec4> as input (hardcoded for speed)
  virtual void setMom(vector<Vec4> pIn, bool permute=true) {
    for (int i=0; i<nLegs(); ++i) {
      if (permute) {
        pSav[i][0] = pIn[iVincia[i]].e();
  	pSav[i][1] = pIn[iVincia[i]].px();
        pSav[i][2] = pIn[iVincia[i]].py();
        pSav[i][3] = pIn[iVincia[i]].pz();
      } else {
        pSav[i][0] = pIn[i].e();
        pSav[i][1] = pIn[i].px();
        pSav[i][2] = pIn[i].py();
        pSav[i][3] = pIn[i].pz();
      }
    }
  }

  // Print momenta (in MG enumeration)
  virtual void printMom() {
    for (int i=0; i<nLegs(); ++i) {
      cout<<" pMG"<<i<<setprecision(6)<<" = ";
      for (int j = 0; j<4; ++j) cout<<" "<<setw(16)<<pSav[i][j];
      cout<<endl;
    }
  }

  // Wrapper for MG5 matrix element squared
  virtual double getME2(int mcMode);

 protected: 

  // Momenta in MadGraph format: p[NLEGS][4]
  double pSav[4][4];

}; // end class MG5dxd_gg

} // end namespace Vincia

#endif

